@@ -47,16 +47,22 @@ std::shared_ptr<arrow::Schema> Table__schema(const std::shared_ptr<arrow::Table>
 }
 
 // [[arrow::export]]
-std::shared_ptr<arrow::Column> Table__column(const std::shared_ptr<arrow::Table>& table,
-                                             int i) {
+std::shared_ptr<arrow::ChunkedArray> Table__column(
+    const std::shared_ptr<arrow::Table>& table, int i) {
   return table->column(i);
 }
 
 // [[arrow::export]]
-std::vector<std::shared_ptr<arrow::Column>> Table__columns(
+std::shared_ptr<arrow::Field> Table__field(const std::shared_ptr<arrow::Table>& table,
+                                           int i) {
+  return table->field(i);
+}
+
+// [[arrow::export]]
+std::vector<std::shared_ptr<arrow::ChunkedArray>> Table__columns(
     const std::shared_ptr<arrow::Table>& table) {
   auto nc = table->num_columns();
-  std::vector<std::shared_ptr<arrow::Column>> res(nc);
+  std::vector<std::shared_ptr<arrow::ChunkedArray>> res(nc);
   for (int i = 0; i < nc; i++) {
     res[i] = table->column(i);
   }
@@ -64,13 +70,32 @@ std::vector<std::shared_ptr<arrow::Column>> Table__columns(
 }
 
 // [[arrow::export]]
-Rcpp::CharacterVector Table__column_names(const std::shared_ptr<arrow::Table>& table) {
-  int nc = table->num_columns();
-  Rcpp::CharacterVector res(nc);
-  for (int i = 0; i < nc; i++) {
-    res[i] = table->column(i)->name();
-  }
-  return res;
+std::vector<std::string> Table__ColumnNames(const std::shared_ptr<arrow::Table>& table) {
+  return table->ColumnNames();
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Table> Table__Slice1(const std::shared_ptr<arrow::Table>& table,
+                                            int offset) {
+  return table->Slice(offset);
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::Table> Table__Slice2(const std::shared_ptr<arrow::Table>& table,
+                                            int offset, int length) {
+  return table->Slice(offset, length);
+}
+
+// [[arrow::export]]
+bool Table__Equals(const std::shared_ptr<arrow::Table>& lhs,
+                   const std::shared_ptr<arrow::Table>& rhs) {
+  return lhs->Equals(*rhs.get());
+}
+
+// [[arrow::export]]
+std::shared_ptr<arrow::ChunkedArray> Table__GetColumnByName(
+    const std::shared_ptr<arrow::Table>& table, const std::string& name) {
+  return table->GetColumnByName(name);
 }
 
 // [[arrow::export]]
@@ -79,7 +104,7 @@ std::shared_ptr<arrow::Table> Table__select(const std::shared_ptr<arrow::Table>&
   R_xlen_t n = indices.size();
 
   std::vector<std::shared_ptr<arrow::Field>> fields(n);
-  std::vector<std::shared_ptr<arrow::Column>> columns(n);
+  std::vector<std::shared_ptr<arrow::ChunkedArray>> columns(n);
 
   for (R_xlen_t i = 0; i < n; i++) {
     int pos = indices[i] - 1;
@@ -94,7 +119,7 @@ std::shared_ptr<arrow::Table> Table__select(const std::shared_ptr<arrow::Table>&
 bool all_record_batches(SEXP lst) {
   R_xlen_t n = XLENGTH(lst);
   for (R_xlen_t i = 0; i < n; i++) {
-    if (!Rf_inherits(VECTOR_ELT(lst, i), "arrow::RecordBatch")) return false;
+    if (!Rf_inherits(VECTOR_ELT(lst, i), "RecordBatch")) return false;
   }
   return true;
 }
@@ -108,7 +133,7 @@ std::shared_ptr<arrow::Table> Table__from_dots(SEXP lst, SEXP schema_sxp) {
     auto batches = arrow::r::List_to_shared_ptr_vector<arrow::RecordBatch>(lst);
     std::shared_ptr<arrow::Table> tab;
 
-    if (Rf_inherits(schema_sxp, "arrow::Schema")) {
+    if (Rf_inherits(schema_sxp, "Schema")) {
       auto schema = arrow::r::extract<arrow::Schema>(schema_sxp);
       STOP_IF_NOT_OK(arrow::Table::FromRecordBatches(schema, batches, &tab));
     } else {
@@ -120,7 +145,7 @@ std::shared_ptr<arrow::Table> Table__from_dots(SEXP lst, SEXP schema_sxp) {
   int num_fields;
   STOP_IF_NOT_OK(arrow::r::count_fields(lst, &num_fields));
 
-  std::vector<std::shared_ptr<arrow::Column>> columns(num_fields);
+  std::vector<std::shared_ptr<arrow::ChunkedArray>> columns(num_fields);
   std::shared_ptr<arrow::Schema> schema;
 
   if (Rf_isNull(schema_sxp)) {
@@ -129,21 +154,18 @@ std::shared_ptr<arrow::Table> Table__from_dots(SEXP lst, SEXP schema_sxp) {
     SEXP names = Rf_getAttrib(lst, R_NamesSymbol);
 
     auto fill_one_column = [&columns, &fields](int j, SEXP x, SEXP name) {
-      if (Rf_inherits(x, "arrow::Column")) {
-        columns[j] = arrow::r::extract<arrow::Column>(x);
-        fields[j] = columns[j]->field();
-      } else if (Rf_inherits(x, "arrow::ChunkedArray")) {
+      if (Rf_inherits(x, "ChunkedArray")) {
         auto chunked_array = arrow::r::extract<arrow::ChunkedArray>(x);
-        fields[j] = std::make_shared<arrow::Field>(CHAR(name), chunked_array->type());
-        columns[j] = std::make_shared<arrow::Column>(fields[j], chunked_array);
-      } else if (Rf_inherits(x, "arrow::Array")) {
+        fields[j] = arrow::field(CHAR(name), chunked_array->type());
+        columns[j] = chunked_array;
+      } else if (Rf_inherits(x, "Array")) {
         auto array = arrow::r::extract<arrow::Array>(x);
-        fields[j] = std::make_shared<arrow::Field>(CHAR(name), array->type());
-        columns[j] = std::make_shared<arrow::Column>(fields[j], array);
+        fields[j] = arrow::field(CHAR(name), array->type());
+        columns[j] = std::make_shared<arrow::ChunkedArray>(array);
       } else {
         auto array = Array__from_vector(x, R_NilValue);
-        fields[j] = std::make_shared<arrow::Field>(CHAR(name), array->type());
-        columns[j] = std::make_shared<arrow::Column>(fields[j], array);
+        fields[j] = arrow::field(CHAR(name), array->type());
+        columns[j] = std::make_shared<arrow::ChunkedArray>(array);
       }
     };
 
@@ -163,23 +185,21 @@ std::shared_ptr<arrow::Table> Table__from_dots(SEXP lst, SEXP schema_sxp) {
     }
 
     schema = std::make_shared<arrow::Schema>(std::move(fields));
-  } else {
+  } else if (Rf_inherits(schema_sxp, "Schema")) {
     // use the schema that is given
     schema = arrow::r::extract<arrow::Schema>(schema_sxp);
 
     auto fill_one_column = [&columns, &schema](int j, SEXP x) {
-      if (Rf_inherits(x, "arrow::Column")) {
-        columns[j] = arrow::r::extract<arrow::Column>(x);
-      } else if (Rf_inherits(x, "arrow::ChunkedArray")) {
+      if (Rf_inherits(x, "ChunkedArray")) {
         auto chunked_array = arrow::r::extract<arrow::ChunkedArray>(x);
-        columns[j] = std::make_shared<arrow::Column>(schema->field(j), chunked_array);
-      } else if (Rf_inherits(x, "arrow::Array")) {
+        columns[j] = chunked_array;
+      } else if (Rf_inherits(x, "Array")) {
         auto array = arrow::r::extract<arrow::Array>(x);
-        columns[j] = std::make_shared<arrow::Column>(schema->field(j), array);
+        columns[j] = std::make_shared<arrow::ChunkedArray>(array);
       } else {
         auto type = schema->field(j)->type();
         auto array = arrow::r::Array__from_vector(x, type, false);
-        columns[j] = std::make_shared<arrow::Column>(schema->field(j), array);
+        columns[j] = std::make_shared<arrow::ChunkedArray>(array);
       }
     };
 
@@ -197,6 +217,8 @@ std::shared_ptr<arrow::Table> Table__from_dots(SEXP lst, SEXP schema_sxp) {
         j++;
       }
     }
+  } else {
+    Rcpp::stop("schema must be an arrow::Schema or NULL");
   }
 
   return arrow::Table::Make(schema, columns);
